import { Observer, Group, CreateObserverConfigInterface, RuntimeJob, Item, IngestConfigInterface, CreateRuntimeJobConfigInterface } from '../../internal';
export declare class GroupObserver<DataType = any> extends Observer {
    group: () => Group<DataType>;
    nextGroupOutput: DataType[];
    /**
     * A Group Observer manages the subscriptions to Subscription Containers (UI-Components)
     * and dependencies to other Observers (Agile Classes)
     * for a Group Class.
     *
     * @internal
     * @param group - Instance of Group the Observer belongs to.
     * @param config - Configuration object
     */
    constructor(group: Group<DataType>, config?: CreateObserverConfigInterface);
    /**
     * Rebuilds the Group and passes the Group Observer
     * into the runtime wrapped into a Runtime-Job
     * where it is executed accordingly.
     *
     * During the execution the runtime applies the rebuilt `nextGroupOutput` to the Group,
     * updates its dependents and re-renders the UI-Components it is subscribed to.
     *
     * @internal
     * @param config - Configuration object
     */
    ingest(config?: GroupIngestConfigInterface): void;
    /**
     * Passes the Group Observer into the runtime wrapped into a Runtime-Job
     * where it is executed accordingly.
     *
     * During the execution the runtime applies the specified `nextGroupOutput` to the Group,
     * updates its dependents and re-renders the UI-Components it is subscribed to.
     *
     * @internal
     * @param newGroupItems - New Group Items to be applied to the Group.
     * @param config - Configuration object.
     */
    ingestItems(newGroupItems: Item<DataType>[], config?: GroupIngestConfigInterface): void;
    /**
     * Method executed by the Runtime to perform the Runtime-Job,
     * previously ingested via the `ingest()` or `ingestItems()` method.
     *
     * Thereby the previously defined `nextGroupOutput` is assigned to the Group.
     *
     * @internal
     * @param job - Runtime-Job to be performed.
     */
    perform(job: RuntimeJob): void;
}
export interface GroupIngestConfigInterface extends CreateRuntimeJobConfigInterface, IngestConfigInterface {
}
