"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selector = void 0;
const internal_1 = require("../internal");
class Selector extends internal_1.State {
    constructor(collection, itemKey, config = {}) {
        config = Object.assign({ isPlaceholder: false }, config);
        super(collection.agileInstance(), null, config);
        this.collection = () => collection;
        this._item = null;
        this._itemKey = !config.isPlaceholder && itemKey != null ? itemKey : null;
        this._key = config === null || config === void 0 ? void 0 : config.key;
        this.isPlaceholder = true;
        if (this._itemKey != null)
            this.select(itemKey, { overwrite: true });
    }
    get itemKey() {
        return this._itemKey;
    }
    set itemKey(value) {
        this.select(value);
    }
    get item() {
        return this._item;
    }
    set item(value) {
        if (value === null || value === void 0 ? void 0 : value._key)
            this.select(value._key);
    }
    select(itemKey, config = {}) {
        var _a, _b;
        config = Object.assign({ background: false, sideEffects: {
                enabled: true,
                exclude: [],
            }, force: false, overwrite: (_b = (_a = this._item) === null || _a === void 0 ? void 0 : _a.isPlaceholder) !== null && _b !== void 0 ? _b : false, storage: true }, config);
        if ((!this.collection().isInstantiated || this.hasSelected(itemKey)) &&
            !config.force)
            return this;
        this.unselect({ background: itemKey != null });
        if (itemKey == null)
            return this;
        const newItem = this.collection().getItemWithReference(itemKey);
        this._itemKey = itemKey;
        this._item = newItem;
        newItem.selectedBy.add(this._key);
        newItem.addSideEffect(Selector.rebuildSelectorSideEffectKey, (instance, config) => this.rebuildSelector(config), { weight: 100 });
        this.addSideEffect(Selector.rebuildItemSideEffectKey, (instance, config) => {
            var _a, _b;
            if (!((_a = instance._item) === null || _a === void 0 ? void 0 : _a.isPlaceholder))
                (_b = instance._item) === null || _b === void 0 ? void 0 : _b.set(instance._value, Object.assign(Object.assign({}, config), {
                    sideEffects: {
                        enabled: true,
                        exclude: [Selector.rebuildSelectorSideEffectKey],
                    },
                }));
        }, { weight: 90 });
        this.rebuildSelector(config);
        return this;
    }
    reselect(config = {}) {
        if ((this._itemKey != null && !this.hasSelected(this._itemKey)) ||
            config.force)
            this.select(this._itemKey, config);
        return this;
    }
    unselect(config = {}) {
        const item = this.collection().getItem(this._itemKey, {
            notExisting: true,
        });
        if (item != null) {
            item.selectedBy.delete(this._key);
            item.removeSideEffect(Selector.rebuildSelectorSideEffectKey);
            item.removeSideEffect(Selector.rebuildItemSideEffectKey);
            if (item.isPlaceholder && this._itemKey != null)
                delete this.collection().data[this._itemKey];
        }
        this._item = null;
        this._itemKey = null;
        this.rebuildSelector(config);
        this.isPlaceholder = true;
        return this;
    }
    hasSelected(itemKey, correctlySelected = true) {
        if (correctlySelected) {
            return (this._itemKey === itemKey &&
                this._item != null &&
                this._item.selectedBy.has(this._key));
        }
        return this._itemKey === itemKey;
    }
    rebuildSelector(config = {}) {
        if (this._item == null || this._item.isPlaceholder) {
            this.set(null, config);
            return this;
        }
        this.set(this._item._value, config);
        return this;
    }
}
exports.Selector = Selector;
Selector.rebuildSelectorSideEffectKey = 'rebuildSelector';
Selector.rebuildItemSideEffectKey = 'rebuildItem';
