"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Computed = void 0;
const internal_1 = require("../internal");
class Computed extends internal_1.State {
    constructor(agileInstance, computeFunction, config = {}) {
        super(agileInstance, null, {
            key: config.key,
            dependents: config.dependents,
        });
        this.deps = new Set();
        this.hardCodedDeps = [];
        config = Object.assign({ computedDeps: [], autodetect: !internal_1.isAsyncFunction(computeFunction) }, config);
        this.agileInstance = () => agileInstance;
        this.computeFunction = computeFunction;
        this.config = {
            autodetect: config.autodetect,
        };
        this.hardCodedDeps = internal_1.extractRelevantObservers(config.computedDeps).filter((dep) => dep !== undefined);
        this.deps = new Set(this.hardCodedDeps);
        this.deps.forEach((observer) => {
            observer.addDependent(this.observers['value']);
        });
        this.recompute({ autodetect: config.autodetect, overwrite: true });
    }
    recompute(config = {}) {
        config = Object.assign({ autodetect: false }, config);
        this.compute({ autodetect: config.autodetect }).then((result) => {
            this.observers['value'].ingestValue(result, internal_1.removeProperties(config, ['autodetect']));
        });
        return this;
    }
    updateComputeFunction(computeFunction, deps = [], config = {}) {
        config = Object.assign({ autodetect: this.config.autodetect }, config);
        this.deps.forEach((observer) => {
            observer.removeDependent(this.observers['value']);
        });
        this.hardCodedDeps = internal_1.extractRelevantObservers(deps).filter((dep) => dep !== undefined);
        this.deps = new Set(this.hardCodedDeps);
        this.deps.forEach((observer) => {
            observer.addDependent(this.observers['value']);
        });
        this.computeFunction = computeFunction;
        this.recompute(internal_1.removeProperties(config, ['overwriteDeps']));
        return this;
    }
    compute(config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            config = Object.assign({ autodetect: this.config.autodetect }, config);
            if (config.autodetect)
                internal_1.ComputedTracker.track();
            const computedValue = this.computeFunction();
            if (config.autodetect) {
                const foundDeps = internal_1.ComputedTracker.getTrackedObservers();
                this.deps.forEach((observer) => {
                    if (!foundDeps.includes(observer) &&
                        !this.hardCodedDeps.includes(observer)) {
                        this.deps.delete(observer);
                        observer.removeDependent(this.observers['value']);
                    }
                });
                foundDeps.forEach((observer) => {
                    if (!this.deps.has(observer)) {
                        this.deps.add(observer);
                        observer.addDependent(this.observers['value']);
                    }
                });
            }
            return computedValue;
        });
    }
    persist() {
        internal_1.LogCodeManager.log('19:03:00');
        return this;
    }
}
exports.Computed = Computed;
