"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Integrations = void 0;
const internal_1 = require("../internal");
const onRegisterInitialIntegrationCallbacks = [];
class Integrations {
    constructor(agileInstance, config = {}) {
        this.integrations = new Set();
        config = Object.assign({ autoIntegrate: true }, config);
        this.agileInstance = () => agileInstance;
        if (config.autoIntegrate) {
            Integrations.onRegisterInitialIntegration((integration) => {
                this.integrate(integration);
            });
        }
    }
    static addInitialIntegration(integration) {
        if (integration instanceof internal_1.Integration) {
            onRegisterInitialIntegrationCallbacks.forEach((callback) => callback(integration));
            Integrations.initialIntegrations.push(integration);
        }
    }
    static onRegisterInitialIntegration(callback) {
        onRegisterInitialIntegrationCallbacks.push(callback);
        Integrations.initialIntegrations.forEach((integration) => {
            callback(integration);
        });
    }
    integrate(integration) {
        return __awaiter(this, void 0, void 0, function* () {
            if (integration._key == null) {
                internal_1.LogCodeManager.log('18:03:00', [integration._key, this.agileInstance().key], integration);
                return false;
            }
            if (integration.methods.bind)
                integration.ready = yield integration.methods.bind(this.agileInstance());
            else
                integration.ready = true;
            this.integrations.add(integration);
            integration.integrated = true;
            internal_1.LogCodeManager.log('18:00:00', [integration._key, this.agileInstance().key], integration);
            return true;
        });
    }
    update(componentInstance, updatedData) {
        this.integrations.forEach((integration) => {
            if (!integration.ready) {
                internal_1.LogCodeManager.log('18:02:00', [integration._key]);
                return;
            }
            if (integration.methods.updateMethod)
                integration.methods.updateMethod(componentInstance, updatedData);
        });
    }
    hasIntegration() {
        return this.integrations.size > 0;
    }
}
exports.Integrations = Integrations;
Integrations.initialIntegrations = [];
