"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubController = void 0;
const internal_1 = require("../../internal");
class SubController {
    constructor(agileInstance) {
        this.componentSubs = new Set();
        this.callbackSubs = new Set();
        this.mountedComponents = new Set();
        this.agileInstance = () => agileInstance;
    }
    subscribe(integrationInstance, subs, config = {}) {
        config = Object.assign({ waitForMount: this.agileInstance().config.waitForMount }, config);
        const subscriptionContainer = internal_1.isFunction(integrationInstance)
            ? this.createCallbackSubscriptionContainer(integrationInstance, subs, config)
            : this.createComponentSubscriptionContainer(integrationInstance, subs, config);
        if (subscriptionContainer.isObjectBased && !Array.isArray(subs)) {
            const props = {};
            for (const key in subs)
                if (subs[key].value)
                    props[key] = subs[key].value;
            return { subscriptionContainer, props };
        }
        return subscriptionContainer;
    }
    unsubscribe(subscriptionInstance) {
        const unsub = (subscriptionContainer) => {
            subscriptionContainer.ready = false;
            subscriptionContainer.subscribers.forEach((observer) => {
                subscriptionContainer.removeSubscription(observer);
            });
        };
        if (subscriptionInstance instanceof internal_1.CallbackSubscriptionContainer) {
            unsub(subscriptionInstance);
            this.callbackSubs.delete(subscriptionInstance);
            internal_1.LogCodeManager.logIfTags(['subscription'], '15:01:00', [], subscriptionInstance);
            return;
        }
        if (subscriptionInstance instanceof internal_1.ComponentSubscriptionContainer) {
            unsub(subscriptionInstance);
            this.componentSubs.delete(subscriptionInstance);
            internal_1.LogCodeManager.logIfTags(['subscription'], '15:01:01', [], subscriptionInstance);
            return;
        }
        if (subscriptionInstance['componentSubscriptionContainers'] !== null &&
            Array.isArray(subscriptionInstance.componentSubscriptionContainers)) {
            subscriptionInstance.componentSubscriptionContainers.forEach((subContainer) => {
                unsub(subContainer);
                this.componentSubs.delete(subContainer);
                internal_1.LogCodeManager.logIfTags(['subscription'], '15:01:01', [], subscriptionInstance);
            });
            return;
        }
    }
    createComponentSubscriptionContainer(componentInstance, subs, config = {}) {
        const componentSubscriptionContainer = new internal_1.ComponentSubscriptionContainer(componentInstance, subs, internal_1.removeProperties(config, ['waitForMount']));
        this.componentSubs.add(componentSubscriptionContainer);
        if (config.waitForMount) {
            if (this.mountedComponents.has(componentInstance))
                componentSubscriptionContainer.ready = true;
        }
        else
            componentSubscriptionContainer.ready = true;
        if (componentInstance['componentSubscriptionContainers'] != null &&
            Array.isArray(componentInstance.componentSubscriptionContainers))
            componentInstance.componentSubscriptionContainers.push(componentSubscriptionContainer);
        else
            componentInstance['componentSubscriptionContainers'] = [
                componentSubscriptionContainer,
            ];
        internal_1.LogCodeManager.logIfTags(['subscription'], '15:01:02', [], componentSubscriptionContainer);
        return componentSubscriptionContainer;
    }
    createCallbackSubscriptionContainer(callbackFunction, subs, config = {}) {
        const callbackSubscriptionContainer = new internal_1.CallbackSubscriptionContainer(callbackFunction, subs, internal_1.removeProperties(config, ['waitForMount']));
        this.callbackSubs.add(callbackSubscriptionContainer);
        callbackSubscriptionContainer.ready = true;
        internal_1.LogCodeManager.logIfTags(['subscription'], '15:01:03', [], callbackSubscriptionContainer);
        return callbackSubscriptionContainer;
    }
    mount(componentInstance) {
        if (componentInstance['componentSubscriptionContainers'] != null &&
            Array.isArray(componentInstance.componentSubscriptionContainers))
            componentInstance.componentSubscriptionContainers.map((c) => (c.ready = true));
        this.mountedComponents.add(componentInstance);
    }
    unmount(componentInstance) {
        if (componentInstance['componentSubscriptionContainers'] != null &&
            Array.isArray(componentInstance.componentSubscriptionContainers))
            componentInstance.componentSubscriptionContainers.map((c) => (c.ready = false));
        this.mountedComponents.delete(componentInstance);
    }
}
exports.SubController = SubController;
