"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createComputed = exports.createCollection = exports.createState = exports.createStorage = exports.assignSharedAgileInstance = exports.shared = void 0;
const internal_1 = require("./internal");
let sharedAgileInstance = new internal_1.Agile({
    key: 'shared',
    localStorage: !internal_1.runsOnServer(),
});
exports.shared = sharedAgileInstance;
function assignSharedAgileInstance(agileInstance) {
    exports.shared = sharedAgileInstance = agileInstance;
}
exports.assignSharedAgileInstance = assignSharedAgileInstance;
function createStorage(config) {
    return new internal_1.Storage(config);
}
exports.createStorage = createStorage;
function createState(initialValue, config = {}) {
    config = Object.assign({ agileInstance: sharedAgileInstance }, config);
    return new internal_1.State(config.agileInstance, initialValue, internal_1.removeProperties(config, ['agileInstance']));
}
exports.createState = createState;
function createCollection(config, agileInstance = sharedAgileInstance) {
    return new internal_1.Collection(agileInstance, config);
}
exports.createCollection = createCollection;
function createComputed(computeFunction, configOrDeps) {
    let _config = {};
    if (Array.isArray(configOrDeps)) {
        _config = internal_1.flatMerge(_config, {
            computedDeps: configOrDeps,
        });
    }
    else {
        if (configOrDeps)
            _config = configOrDeps;
    }
    _config = Object.assign({ agileInstance: sharedAgileInstance }, _config);
    return new internal_1.Computed(_config.agileInstance, computeFunction, internal_1.removeProperties(_config, ['agileInstance']));
}
exports.createComputed = createComputed;
