"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatePersistent = void 0;
const internal_1 = require("../internal");
class StatePersistent extends internal_1.Persistent {
    constructor(state, config = {}) {
        super(state.agileInstance(), {
            instantiate: false,
        });
        config = Object.assign({ instantiate: true, storageKeys: [], defaultStorageKey: null }, config);
        this.state = () => state;
        this.instantiatePersistent({
            key: config.key,
            storageKeys: config.storageKeys,
            defaultStorageKey: config.defaultStorageKey,
        });
        if (this.ready && config.instantiate)
            this.initialLoading();
    }
    initialLoading() {
        const _super = Object.create(null, {
            initialLoading: { get: () => super.initialLoading }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.initialLoading.call(this).then(() => {
                this.state().isPersisted = true;
            });
        });
    }
    loadPersistedValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
            const loadedValue = yield this.agileInstance().storages.get(_storageItemKey, this.config.defaultStorageKey);
            if (loadedValue == null)
                return false;
            this.state().set(loadedValue, {
                storage: false,
                overwrite: true,
            });
            this.setupSideEffects(_storageItemKey);
            return true;
        });
    }
    persistValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
            this.setupSideEffects(_storageItemKey);
            this.rebuildStorageSideEffect(this.state(), _storageItemKey);
            this.isPersisted = true;
            return true;
        });
    }
    setupSideEffects(storageItemKey) {
        const _storageItemKey = storageItemKey !== null && storageItemKey !== void 0 ? storageItemKey : this._key;
        this.state().addSideEffect(StatePersistent.storeValueSideEffectKey, (instance, config) => {
            this.rebuildStorageSideEffect(this.state(), _storageItemKey, config);
        }, { weight: 0 });
    }
    removePersistedValue(storageItemKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const _storageItemKey = storageItemKey || this._key;
            this.state().removeSideEffect(StatePersistent.storeValueSideEffectKey);
            this.agileInstance().storages.remove(_storageItemKey, this.storageKeys);
            this.isPersisted = false;
            return true;
        });
    }
    formatKey(key) {
        if (key == null && this.state()._key)
            return this.state()._key;
        if (key == null)
            return;
        if (this.state()._key == null)
            this.state()._key = key;
        return key;
    }
    rebuildStorageSideEffect(state, storageItemKey, config = {}) {
        if (config['storage'] == null || config.storage) {
            this.agileInstance().storages.set(storageItemKey, this.state().getPersistableValue(), this.storageKeys);
        }
    }
}
exports.StatePersistent = StatePersistent;
StatePersistent.storeValueSideEffectKey = 'rebuildStateStorageValue';
